@@ -24,8 +24,10 @@ from os_win import exceptions
 from os_win.tests import test_base
 from os_win.utils.compute import clusterutils
 from os_win.utils.compute import livemigrationutils
+from os_win.utils.compute import migrationutils
 from os_win.utils.compute import rdpconsoleutils
 from os_win.utils.compute import vmutils
+from os_win.utils.dns import dnsutils
 from os_win.utils import hostutils
 from os_win.utils.network import networkutils
 from os_win.utils import pathutils
@@ -140,3 +142,13 @@ class TestHyperVUtilsFactory(test_base.OsWinBaseTestCase):
         self._check_get_class(
             expected_class=clusterutils.ClusterUtils,
             class_type='clusterutils')
+
+    def test_get_dnsutils(self):
+        self._check_get_class(
+            expected_class=dnsutils.DNSUtils,
+            class_type='dnsutils')
+
+    def test_get_migrationutils(self):
+        self._check_get_class(
+            expected_class=migrationutils.MigrationUtils,
+            class_type='migrationutils')
