@@ -21,16 +21,11 @@ Hyper-V Server / Windows Server 2012.
 """
 
 import functools
-import sys
 import time
 import uuid
 
-if sys.platform == 'win32':
-    import wmi
-
 from eventlet import patcher
 from eventlet import tpool
-from oslo_config import cfg
 from oslo_log import log as logging
 from oslo_utils import uuidutils
 import six
@@ -45,7 +40,6 @@ from os_win.utils import baseutils
 from os_win.utils import jobutils
 from os_win.utils import pathutils
 
-CONF = cfg.CONF
 LOG = logging.getLogger(__name__)
 
 
@@ -85,6 +79,7 @@ class VMUtils(baseutils.BaseUtilsVirt):
 
     _VIRTUAL_SYSTEM_SUBTYPE = 'VirtualSystemSubType'
     _VIRTUAL_SYSTEM_TYPE_REALIZED = 'Microsoft:Hyper-V:System:Realized'
+    _VIRTUAL_SYSTEM_TYPE_PLANNED = 'Microsoft:Hyper-V:System:Planned'
     _VIRTUAL_SYSTEM_SUBTYPE_GEN2 = 'Microsoft:Hyper-V:SubType:2'
 
     _SNAPSHOT_FULL = 2
@@ -127,7 +122,7 @@ class VMUtils(baseutils.BaseUtilsVirt):
 
     def __init__(self, host='.'):
         super(VMUtils, self).__init__(host)
-        self._jobutils = jobutils.JobUtils()
+        self._jobutils = jobutils.JobUtils(host)
         self._pathutils = pathutils.PathUtils()
         self._enabled_states_map = {v: k for k, v in
                                     six.iteritems(self._vm_power_states_map)}
@@ -148,6 +143,7 @@ class VMUtils(baseutils.BaseUtilsVirt):
 
     def list_instances(self):
         """Return the names of all the instances known to Hyper-V."""
+
         return [v.ElementName for v in
                 self._conn.Msvm_VirtualSystemSettingData(
                     ['ElementName'],
@@ -182,7 +178,8 @@ class VMUtils(baseutils.BaseUtilsVirt):
         # considering that in all the non mappable states the instance
         # is running.
         enabled_state = self._enabled_states_map.get(si.EnabledState,
-            constants.HYPERV_VM_STATE_ENABLED)
+                                                     constants.
+                                                     HYPERV_VM_STATE_ENABLED)
 
         summary_info_dict = {'NumberOfProcessors': si.NumberOfProcessors,
                              'EnabledState': enabled_state,
@@ -194,18 +191,21 @@ class VMUtils(baseutils.BaseUtilsVirt):
         settings = self.get_vm_summary_info(vm_name)
         return settings['EnabledState']
 
-    def _lookup_vm_check(self, vm_name, as_vssd=True, for_update=False):
-        vm = self._lookup_vm(vm_name, as_vssd, for_update)
+    def _lookup_vm_check(self, vm_name, as_vssd=True, for_update=False,
+                         virtual_system_type=_VIRTUAL_SYSTEM_TYPE_REALIZED):
+        vm = self._lookup_vm(vm_name, as_vssd, for_update,
+                             virtual_system_type)
         if not vm:
             raise exceptions.HyperVVMNotFoundException(vm_name=vm_name)
         return vm
 
-    def _lookup_vm(self, vm_name, as_vssd=True, for_update=False):
+    def _lookup_vm(self, vm_name, as_vssd=True, for_update=False,
+                   virtual_system_type=_VIRTUAL_SYSTEM_TYPE_REALIZED):
         if as_vssd:
             conn = self._compat_conn if for_update else self._conn
             vms = conn.Msvm_VirtualSystemSettingData(
                 ElementName=vm_name,
-                VirtualSystemType=self._VIRTUAL_SYSTEM_TYPE_REALIZED)
+                VirtualSystemType=virtual_system_type)
         else:
             vms = self._conn.Msvm_ComputerSystem(ElementName=vm_name)
         n = len(vms)
@@ -271,12 +271,24 @@ class VMUtils(baseutils.BaseUtilsVirt):
         self._jobutils.modify_virt_resource(procsetting)
 
     def update_vm(self, vm_name, memory_mb, memory_per_numa_node, vcpus_num,
-                  vcpus_per_numa_node, limit_cpu_features, dynamic_mem_ratio):
-        vmsetting = self._lookup_vm_check(vm_name)
+                  vcpus_per_numa_node, limit_cpu_features, dynamic_mem_ratio,
+                  configuration_root_dir=None, snapshot_dir=None,
+                  is_planned_vm=False):
+        virtual_system_type = self._get_virtual_system_type(is_planned_vm)
+
+        vmsetting = self._lookup_vm_check(
+            vm_name, virtual_system_type=virtual_system_type)
+
+        if configuration_root_dir:
+            vmsetting.ConfigurationDataRoot = configuration_root_dir
+        if snapshot_dir:
+            vmsetting.SnapshotDataRoot = snapshot_dir
         self._set_vm_memory(vmsetting, memory_mb, memory_per_numa_node,
                             dynamic_mem_ratio)
         self._set_vm_vcpus(vmsetting, vcpus_num, vcpus_per_numa_node,
                            limit_cpu_features)
+        if configuration_root_dir or snapshot_dir:
+            self._modify_virtual_system(vmsetting)
 
     def check_admin_permissions(self):
         if not self._compat_conn.Msvm_VirtualSystemManagementService():
@@ -370,8 +382,8 @@ class VMUtils(baseutils.BaseUtilsVirt):
             self._conn, self._RESOURCE_ALLOC_SETTING_DATA_CLASS,
             element_instance_id=vmsettings.InstanceID)
         ide_ctrls = [r for r in rasds
-                     if r.ResourceSubType == self._IDE_CTRL_RES_SUB_TYPE
-                     and r.Address == str(ctrller_addr)]
+                     if r.ResourceSubType == self._IDE_CTRL_RES_SUB_TYPE and
+                     r.Address == str(ctrller_addr)]
 
         return ide_ctrls[0].path_() if ide_ctrls else None
 
@@ -492,9 +504,10 @@ class VMUtils(baseutils.BaseUtilsVirt):
             diskdrive.ElementName = serial
             self._jobutils.modify_virt_resource(diskdrive)
 
-    def get_vm_physical_disk_mapping(self, vm_name):
+    def get_vm_physical_disk_mapping(self, vm_name, is_planned_vm=False):
         mapping = {}
-        physical_disks = self.get_vm_disks(vm_name)[1]
+        physical_disks = (
+            self.get_vm_disks(vm_name, is_planned_vm=is_planned_vm)[1])
         for diskdrive in physical_disks:
             mapping[diskdrive.ElementName] = dict(
                 resource_path=diskdrive.path_(),
@@ -523,7 +536,7 @@ class VMUtils(baseutils.BaseUtilsVirt):
                 if (disk_resource.HostResource and
                         disk_resource.HostResource[0] != mounted_disk_path):
                     LOG.debug('Updating disk host resource "%(old)s" to '
-                                '"%(new)s"' %
+                              '"%(new)s"' %
                               {'old': disk_resource.HostResource[0],
                                'new': mounted_disk_path})
                     disk_resource.HostResource = [mounted_disk_path]
@@ -585,6 +598,7 @@ class VMUtils(baseutils.BaseUtilsVirt):
 
     def set_vm_state(self, vm_name, req_state):
         """Set the desired state of the VM."""
+
         vm = self._lookup_vm_check(vm_name, as_vssd=False)
         (job_path,
          ret_val) = vm.RequestStateChange(self._vm_power_states_map[req_state])
@@ -598,8 +612,15 @@ class VMUtils(baseutils.BaseUtilsVirt):
     def _get_disk_resource_disk_path(self, disk_resource):
         return disk_resource.HostResource
 
-    def get_vm_storage_paths(self, vm_name):
+    def get_vm_config_root_dir(self, vm_name):
         vmsettings = self._lookup_vm_check(vm_name)
+        return vmsettings.ConfigurationDataRoot
+
+    def get_vm_storage_paths(self, vm_name, is_planned_vm=False):
+        virtual_system_type = self._get_virtual_system_type(is_planned_vm)
+        vmsettings = self._lookup_vm_check(
+            vm_name,
+            virtual_system_type=virtual_system_type)
         (disk_resources, volume_resources) = self._get_vm_disks(vmsettings)
 
         volume_drives = []
@@ -614,8 +635,10 @@ class VMUtils(baseutils.BaseUtilsVirt):
 
         return (disk_files, volume_drives)
 
-    def get_vm_disks(self, vm_name):
-        vmsettings = self._lookup_vm_check(vm_name)
+    def get_vm_disks(self, vm_name, is_planned_vm=False):
+        virtual_system_type = self._get_virtual_system_type(is_planned_vm)
+        vmsettings = self._lookup_vm_check(
+            vm_name, virtual_system_type=virtual_system_type)
         return self._get_vm_disks(vmsettings)
 
     def _get_vm_disks(self, vmsettings):
@@ -800,10 +823,12 @@ class VMUtils(baseutils.BaseUtilsVirt):
 
     def get_active_instances(self):
         """Return the names of all the active instances known to Hyper-V."""
+
         vm_names = self.list_instances()
         vms = [self._lookup_vm(vm_name, as_vssd=False) for vm_name in vm_names]
         active_vm_names = [v.ElementName for v in vms
-            if v.EnabledState == constants.HYPERV_VM_STATE_ENABLED]
+                           if v.EnabledState ==
+                           constants.HYPERV_VM_STATE_ENABLED]
 
         return active_vm_names
 
@@ -851,7 +876,7 @@ class VMUtils(baseutils.BaseUtilsVirt):
                         LOG.exception(err_msg,
                                       dict(vm_name=vm_name,
                                            vm_power_state=vm_power_state))
-                except wmi.x_wmi_timed_out:
+                except exceptions.x_wmi_timed_out:
                     pass
                 except Exception:
                     LOG.exception(
@@ -873,15 +898,16 @@ class VMUtils(baseutils.BaseUtilsVirt):
                                     object transitioned into one of those
                                     states.
         """
+
         query = ("SELECT %(field)s, TargetInstance "
                  "FROM __InstanceModificationEvent "
                  "WITHIN %(timeframe)s "
                  "WHERE TargetInstance ISA '%(class)s' "
                  "AND TargetInstance.%(field)s != "
                  "PreviousInstance.%(field)s" %
-                    {'class': cls,
-                     'field': field,
-                     'timeframe': timeframe})
+                 {'class': cls,
+                  'field': field,
+                  'timeframe': timeframe})
         if filtered_states:
             checks = ["TargetInstance.%s = '%s'" % (field, state)
                       for state in filtered_states]
@@ -928,6 +954,7 @@ class VMUtils(baseutils.BaseUtilsVirt):
                                  Authority for Secure Boot. Only Linux
                                  guests require this CA.
         """
+
         vs_data = self._lookup_vm_check(vm_name)
         self._set_secure_boot(vs_data, msft_ca_required)
         self._modify_virtual_system(vs_data)
@@ -985,7 +1012,7 @@ class VMUtils(baseutils.BaseUtilsVirt):
 
     def _set_boot_order_gen2(self, vm_name, device_boot_order):
         new_boot_order = [(self._drive_to_boot_source(device))
-                           for device in device_boot_order if device]
+                          for device in device_boot_order if device]
 
         vssd = self._lookup_vm_check(vm_name)
         old_boot_order = vssd.BootSourceOrder
@@ -1003,10 +1030,12 @@ class VMUtils(baseutils.BaseUtilsVirt):
 
     def vm_gen_supports_remotefx(self, vm_gen):
         """RemoteFX is supported only for generation 1 virtual machines
+
         on Windows 8 / Windows Server 2012 and 2012R2.
 
         :returns: True if the given vm_gen is 1, False otherwise
         """
+
         return vm_gen == constants.VM_GEN_1
 
     def _validate_remotefx_params(self, monitor_count, max_resolution,
@@ -1021,10 +1050,10 @@ class VMUtils(baseutils.BaseUtilsVirt):
                 _("Unsuported RemoteFX monitor count: %(count)s for "
                   "this resolution %(res)s. Hyper-V supports a maximum "
                   "of %(max_monitors)s monitors for this resolution.")
-                  % {'count': monitor_count,
-                     'res': max_resolution,
-                     'max_monitors': self._remotefx_max_monitors_map[
-                        max_resolution]})
+                % {'count': monitor_count,
+                   'res': max_resolution,
+                   'max_monitors':
+                   self._remotefx_max_monitors_map[max_resolution]})
 
     def _add_3d_display_controller(self, vm, monitor_count,
                                    max_resolution, vram_bytes=None):
@@ -1071,3 +1100,14 @@ class VMUtils(baseutils.BaseUtilsVirt):
 
     def is_secure_vm(self, instance_name):
         return False
+
+    def update_vm_disk_path(self, disk_path, new_disk_path, is_physical=True):
+        disk_resource = self._get_mounted_disk_resource_from_path(
+            disk_path=disk_path, is_physical=is_physical)
+        disk_resource.HostResource = [new_disk_path]
+        self._jobutils.modify_virt_resource(disk_resource)
+
+    def _get_virtual_system_type(self, is_planned_vm):
+        return (
+            self._VIRTUAL_SYSTEM_TYPE_PLANNED if is_planned_vm
+            else self._VIRTUAL_SYSTEM_TYPE_REALIZED)
