@@ -19,11 +19,6 @@ import os
 import shutil
 import sys
 import tempfile
-import time
-
-if sys.platform == 'win32':
-    from ctypes import wintypes
-    kernel32 = ctypes.windll.kernel32
 
 from oslo_log import log as logging
 from oslo_utils import fileutils
@@ -33,6 +28,10 @@ from os_win._i18n import _
 from os_win import exceptions
 from os_win.utils import win32utils
 
+if sys.platform == 'win32':
+    from ctypes import wintypes
+    kernel32 = ctypes.windll.kernel32
+
 LOG = logging.getLogger(__name__)
 
 ERROR_DIR_IS_NOT_EMPTY = 145
@@ -61,6 +60,9 @@ class PathUtils(object):
     def rename(self, src, dest):
         os.rename(src, dest)
 
+    def copy_dir(self, src, dest):
+        shutil.copytree(src, dest)
+
     def copyfile(self, src, dest):
         self.copy(src, dest)
 
@@ -90,6 +92,7 @@ class PathUtils(object):
 
     def move_folder_files(self, src_dir, dest_dir):
         """Moves the files of the given src_dir to dest_dir.
+
         It will ignore any nested folders.
 
         :param src_dir: Given folder from which to move files.
@@ -103,17 +106,7 @@ class PathUtils(object):
                 self.rename(src, os.path.join(dest_dir, fname))
 
     def rmtree(self, path):
-        # This will be removed once support for Windows Server 2008R2 is
-        # stopped
-        for i in range(5):
-            try:
-                shutil.rmtree(path)
-                return
-            except WindowsError as e:
-                if e.winerror == ERROR_DIR_IS_NOT_EMPTY:
-                    time.sleep(1)
-                else:
-                    raise e
+        shutil.rmtree(path)
 
     def check_create_dir(self, path):
         if not self.exists(path):
@@ -142,6 +135,7 @@ class PathUtils(object):
 
         NOTE: Juctions only work on same filesystem.
         """
+
         create_symlink = kernel32.CreateSymbolicLinkW
         create_symlink.argtypes = (
             ctypes.c_wchar_p,
@@ -164,6 +158,7 @@ class PathUtils(object):
     @contextlib.contextmanager
     def temporary_file(self, suffix=None, *args, **kwargs):
         """Creates a random, temporary, closed file, returning the file's
+
         path. It's different from tempfile.NamedTemporaryFile which returns
         an open file descriptor.
         """
